// ------------------------------------------------------------------
//                                                      Utility.mq4
//                                   Copyright  2009, Stefan Stemp
//                                      http://www.stefan-stemp.com
// ------------------------------------------------------------------

#property copyright "Copyright  2009, Stefan Stemp"
#property link      "http://www.stefan-stemp.com"


// ---------------------------------------------------------------------------------------------
// Function P2P() <pip to points>
// ---------------------------------------------------------------------------------------------

double P2P(double pips)
{
   return (P2PV(pips, Point, Digits));
}

// ---------------------------------------------------------------------------------------------
// Function P2PV() <pip to points>
// ---------------------------------------------------------------------------------------------

double P2PV(double pips, int point, int digits)
{
   switch(digits)
   {
   case 2:
      return (NormalizeDouble(point * pips, digits));
   case 3:
      return (NormalizeDouble(point * pips * 10, digits));
   case 4:
      return (NormalizeDouble(point * pips, digits));
   case 5:
      return (NormalizeDouble(point * pips * 10, digits));
   }
   return (0);
}

// ---------------------------------------------------------------------------------------------
// Function Price2Pip()
// ---------------------------------------------------------------------------------------------

double Price2Pip(double price)
{
   switch(MarketInfo(OrderSymbol(), MODE_DIGITS))
   {
      case 2: return (price * 100);
      case 3: return (price * 100);
      case 4: return (price * 10000);
      case 5: return (price * 10000);
   }
   return (0);
}

// ---------------------------------------------------------------------------------------------
// Function NS()
// ---------------------------------------------------------------------------------------------

string NS(double v)
{
   return (DoubleToStr(v, MarketInfo(Symbol(), MODE_DIGITS)));
}

// ---------------------------------------------------------------------------------------------
// Function ND()
// ---------------------------------------------------------------------------------------------

double ND(double v)
{
   return (NormalizeDouble(v, MarketInfo(Symbol(), MODE_DIGITS)));
}

// ---------------------------------------------------------------------------------------------
// Function MaxValue()
// ---------------------------------------------------------------------------------------------

double MaxValue(double v1, double v2)
{
   return (iif(v1 > v2, v1, v2));   
}

// ---------------------------------------------------------------------------------------------
// Function MinValue()
// ---------------------------------------------------------------------------------------------

double MinValue(double v1, double v2)
{
   return (iif(v1 < v2, v1, v2));   
}

// ---------------------------------------------------------------------------------------------
// Function OrderDirection()
// ---------------------------------------------------------------------------------------------

int OrderDirection(int type)
{
   switch(type)
   {
      case OP_BUY:
      case OP_BUYLIMIT:
      case OP_BUYSTOP:
         return (1);
      case OP_SELL:
      case OP_SELLLIMIT:
      case OP_SELLSTOP:
         return (-1);         
   }
   return (0);
}

// ---------------------------------------------------------------------------------------------
// Function iif()
// ---------------------------------------------------------------------------------------------

double iif(bool exp, double vt, double vf)
{
   if (exp)
      return (vt);
   else
      return (vf);
}

// ---------------------------------------------------------------------------------------------
// Function Offset()
// ---------------------------------------------------------------------------------------------

double Offset(double offset)
{
   return (iif(OrderDirection(OrderType()) > 0, P2P(offset), -P2P(offset)));   
}

// ---------------------------------------------------------------------------------------------
// Function Diff()
// ---------------------------------------------------------------------------------------------

double Diff(double p1, double p2)
{
   return (iif(OrderDirection(OrderType()) > 0, p1 - p2, p2 - p1));   
}

// ---------------------------------------------------------------------------------------------
// Function Max()
// ---------------------------------------------------------------------------------------------

double Max(int bar)
{
   return (iif(OrderDirection(OrderType()) > 0, High[bar], Low[bar]));   
}

// ---------------------------------------------------------------------------------------------
// Function Min()
// ---------------------------------------------------------------------------------------------

double Min(int bar)
{
   return (iif(OrderDirection(OrderType()) > 0, Low[bar], High[bar]));   
}

// ---------------------------------------------------------------------------------------------
// Function PriceOpen()
// ---------------------------------------------------------------------------------------------

double PriceOpen(int type)
{
   return (iif(OrderDirection(type) > 0, MarketInfo(Symbol(), MODE_ASK), MarketInfo(Symbol(), MODE_BID)));
}

// ---------------------------------------------------------------------------------------------
// Function PriceClose()
// ---------------------------------------------------------------------------------------------

double PriceClose(int type)
{
   return (iif(OrderDirection(type) > 0, MarketInfo(OrderSymbol(), MODE_BID), MarketInfo(OrderSymbol(), MODE_ASK)));
}

// ---------------------------------------------------------------------------------------------
// Function ColorOpen()
// ---------------------------------------------------------------------------------------------

color ColorOpen(int type)
{
   return (iif(OrderDirection(type) > 0, Blue, Red));
}

// ---------------------------------------------------------------------------------------------
// Function ColorClose()
// ---------------------------------------------------------------------------------------------

color ColorClose(int type)
{
   return (iif(OrderDirection(type) > 0, Blue, Red));
}

// ---------------------------------------------------------------------------------------------
// Function CalcLotsVolume()
// ---------------------------------------------------------------------------------------------

double CalcLotsVolume(double risk, double max)
{
   double freeMargin = AccountFreeMargin();
   if (freeMargin > 0)
   {
      double reqMargin = MarketInfo(Symbol(), MODE_MARGINREQUIRED);         
      double lotStep = MarketInfo(Symbol(), MODE_LOTSTEP);
      double lrr = risk / 100.0;
      double lots = NormalizeDouble(MathFloor(freeMargin * lrr / (reqMargin * lotStep)) * lotStep, 2);
      double orderLots = NormalizeVolume(lots);
   }
   if (orderLots > max)
   {
      orderLots = NormalizeVolume(max);
   }
   return (orderLots);
}

// ---------------------------------------------------------------------------------------------
// Function NormalizeVolume()
// ---------------------------------------------------------------------------------------------

double NormalizeVolume(double volume)
{
   double lotStep = MarketInfo(Symbol(), MODE_LOTSTEP);
   double lotMin = MarketInfo(Symbol(), MODE_MINLOT);
   double lotMax = MarketInfo(Symbol(), MODE_MAXLOT);
   
   if (lotMin == 0.0) lotMin = 0.01;
   if (lotMax == 0.0) lotMax = 50;
   
   double step = 1 / lotStep;   
   double lots = MathFloor(volume * step) / step;
   
   if (lots < lotMin) lots = lotMin;
   if (lots > lotMax) lots = lotMax;
   
   return (lots);
}

// ---------------------------------------------------------------------------------------------
// Function InitVar()
// ---------------------------------------------------------------------------------------------

void InitVar(string name, double value)
{
   // Sets a global variable if not existing yet
   if (!GlobalVariableCheck(name))
   {
      SetVar(name, value);
   }
}

// ---------------------------------------------------------------------------------------------
// Function GetVar()
// ---------------------------------------------------------------------------------------------

void SetVar(string name, double value)
{
   datetime t = GlobalVariableSet(name, value);
   if (t == 0)
   {
      Print("Failed to set global variable '", name, "', error ", GetLastError());
   }   
}

// ---------------------------------------------------------------------------------------------
// Function GetVar()
// ---------------------------------------------------------------------------------------------

double GetVar(string name)
{
   return (GlobalVariableGet(name));
}

// ---------------------------------------------------------------------------------------------
// Function SetLabel()
// ---------------------------------------------------------------------------------------------

void SetLabel(int win, string labelName, string labelText, int labelXDistance, int labelYDistance, color labelColor)
{
   //ObjectDelete(labelName);
   ObjectCreate(labelName, OBJ_LABEL, win, 0, 0);
   ObjectSet(labelName, OBJPROP_CORNER, 0);
   ObjectSet(labelName, OBJPROP_XDISTANCE, labelXDistance);
   ObjectSet(labelName, OBJPROP_YDISTANCE, labelYDistance);
   ObjectSetText(labelName, labelText, 8, "Arial", labelColor);
}
  
// ---------------------------------------------------------------------------------------------
// Function SetText()
// ---------------------------------------------------------------------------------------------

void SetText(string labelName, string labelText = " ", color labelColor = Black)
{
   ObjectSetText(labelName, labelText, 8, "Arial", labelColor);
}
  
// ---------------------------------------------------------------------------------------------
// Function GetOrderTypeText()
// ---------------------------------------------------------------------------------------------

string GetOrderTypeText(int orderType)
{
   switch(orderType)
   {
      case OP_BUY:			return("Buy");
      case OP_SELL:			return("Sell");
      case OP_BUYLIMIT:		return("Buy Limit");
      case OP_BUYSTOP:		return("Buy Stop");
      case OP_SELLLIMIT:   return("Sell Limit");
      case OP_SELLSTOP:		return("Sell Stop");
      default:					return("UnknownOrderType");
   }
}

// ---------------------------------------------------------------------------------------------
// Function GetPeriodText()
// ---------------------------------------------------------------------------------------------

string GetPeriodText(int period)
{
   switch(period)
   {
   case PERIOD_M1:
      return ("M1");
   case PERIOD_M5:
      return ("M5");
   case PERIOD_M15:
      return ("M15");
   case PERIOD_M30:
      return ("M30");
   case PERIOD_H1:
      return ("H1");
   case PERIOD_H4:
      return ("H4");
   case PERIOD_D1:
      return ("D1");
   case PERIOD_W1:
      return ("W1");
   case PERIOD_MN1:
      return ("MN1");
   default:
      return("User [M" + period + "]");
   }
}


