// ------------------------------------------------------------------
//                                                    Pip Watch.mq4
//                                   Copyright  2009, Stefan Stemp
//                                      http://www.stefan-stemp.com
// ------------------------------------------------------------------

#property indicator_chart_window

#include <utility.mqh>

extern color TextColor = YellowGreen;
extern int PositionX = 10;
extern int PositionY = 100;
extern int FontSize  = 8;
extern int Corner = 0;

int orderCount = 0;
int orderTickets[];

double sp = 0.0;
double dp = 0.0;
double wp = 0.0;
double mp = 0.0;
double tp = 0.0;

double sm = 0.0;
double dm = 0.0;
double wm = 0.0;
double mm = 0.0;
double tm = 0.0;


// ---------------------------------------------------------------------------------------------
// Function init()
// ---------------------------------------------------------------------------------------------

int init()
{
  return(0);
}

// ---------------------------------------------------------------------------------------------
// Function deinit()
// ---------------------------------------------------------------------------------------------

int deinit()
{
	ObjectDelete("OpenLabel");
	ObjectDelete("DailyLabel");
	ObjectDelete("WeeklyLabel");
	ObjectDelete("MonthlyLabel");
	ObjectDelete("TotalLabel");

   return(0);
}

// ---------------------------------------------------------------------------------------------
// Function start()
// ---------------------------------------------------------------------------------------------

int start()
{
   sp = 0.0;
   dp = 0.0;
   wp = 0.0;
   mp = 0.0;
   tp = 0.0;
   
   sm = 0.0;
   dm = 0.0;
   wm = 0.0;
   mm = 0.0;
   tm = 0.0;
   
   orderCount = OrdersHistoryTotal() + OrdersTotal();
   
   ArrayResize(orderTickets, orderCount);
   ArrayInitialize(orderTickets, 0);
   
   int line = 0;
   
   // For the purpose of not counting to much points for splitted orders, we have to
   // accumulate all order tickets into an array and if the path of the original order
   // among all splitted orders is followed the orders are taken out from the array.
   
   // Write all closed orders
   for(int i = 0; i < OrdersHistoryTotal(); i++)
   {
      OrderSelect(i, SELECT_BY_POS, MODE_HISTORY);
      orderTickets[i] = OrderTicket();
   }

   // Write all open orders
   for(i = 0; i < OrdersTotal(); i++)
   {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      orderTickets[i + OrdersHistoryTotal()] = OrderTicket();
   }
   
   for(i = 0; i < orderCount; i++)
   {
      // If the order was counted already in case of a splitted order continue with the next
      if (orderTickets[i] == 0)
      {
         continue;
      }
      
      OrderSelect(orderTickets[i], SELECT_BY_TICKET);
      
      // Don't list deposits and withdrawals
      if (OrderSymbol() == "")
      {
         continue;
      }
      
      double pips = Price2Pip(Diff(OrderClosePrice(), OrderOpenPrice()));
      double profit = OrderProfit();
      
      AddOrderPoints(pips, profit);
       
      // In case of a splitted order the profit points of this order must exceed the profit points
      // of the partially closed order to be added to the statistics, otherwise the total amount of
      // points would be incorrect
      while(StringFind(OrderComment(), "to") != -1)
      {
         // Find the order ticket in the commet string
         int start = StringFind(OrderComment(), "#");
         int end = StringFind(OrderComment(), "[", start);
         
         // In case of SL or TP hit the string does end with [tp] or [sl]. In each other case the string
         // ends with the order ticket number
         if (end == -1)
         {
            end = StringLen(OrderComment());
         }
         
         // Extract the order ticket of the original order from which the current order was splitted
         if(start > 0 && end > 0)
         {
            start = start + 1;
            
            string strOrder = StringSubstr(OrderComment(), start, end - start);
            int order = StrToInteger(strOrder);            
            
            // If the order was counted already in case of a splitted order continue with the next
            int index = FindOrderTicket(order, i);      
            if (index < 0)
            {
               break;
            }

            OrderSelect(order, SELECT_BY_TICKET);
            
            // Check the necessary profit points required to be added to the statistic
            if (Price2Pip(Diff(OrderClosePrice(), OrderOpenPrice())) > pips)
            {
               pips = Price2Pip(Diff(OrderClosePrice(), OrderOpenPrice())) - pips;
               profit = OrderProfit();
               
               AddOrderPoints(pips, profit);
            }
            
            // Remove handled order from list
            orderTickets[index] = 0;
            
         }         
      }
            
      // Remove handled order from list
      orderTickets[i] = 0;
   }
      
   ObjectCreate("OpenLabel", OBJ_LABEL, 0, 0.0, 0.0);
   ObjectSetText("OpenLabel", "Open   " + DoubleToStr(sp, 0) + " (" + DoubleToStr(sm, 2) + ")", FontSize, "Arial", TextColor);
   ObjectSet("OpenLabel", OBJPROP_XDISTANCE, PositionX);
   ObjectSet("OpenLabel", OBJPROP_YDISTANCE, PositionY + line * 1.5 * FontSize);
   ObjectSet("OpenLabel", OBJPROP_CORNER, Corner);
   
   line++;
   
   ObjectCreate("DailyLabel", OBJ_LABEL, 0, 0.0, 0.0);
   ObjectSetText("DailyLabel", "Today " + DoubleToStr(dp, 0) + " (" + DoubleToStr(dm, 2) + ")", FontSize, "Arial", TextColor);
   ObjectSet("DailyLabel", OBJPROP_XDISTANCE, PositionX);
   ObjectSet("DailyLabel", OBJPROP_YDISTANCE, PositionY + line * 1.5 * FontSize);
   ObjectSet("DailyLabel", OBJPROP_CORNER, Corner);
 
   line++;

   ObjectCreate("WeeklyLabel", OBJ_LABEL, 0, 0.0, 0.0);
   ObjectSetText("WeeklyLabel", "Week   " + DoubleToStr(wp, 0) + " (" + DoubleToStr(wm, 2) + ")", FontSize, "Arial", TextColor);
   ObjectSet("WeeklyLabel", OBJPROP_XDISTANCE, PositionX);
   ObjectSet("WeeklyLabel", OBJPROP_YDISTANCE, PositionY + line * 1.5 * FontSize);
   ObjectSet("WeeklyLabel", OBJPROP_CORNER, Corner);

   line++;

   ObjectCreate("MonthlyLabel", OBJ_LABEL, 0, 0.0, 0.0);
   ObjectSetText("MonthlyLabel", "Month " + DoubleToStr(mp, 0) + " (" + DoubleToStr(mm, 2) + ")", FontSize, "Arial", TextColor);
   ObjectSet("MonthlyLabel", OBJPROP_XDISTANCE, PositionX);
   ObjectSet("MonthlyLabel", OBJPROP_YDISTANCE, PositionY + line * 1.5 * FontSize);
   ObjectSet("MonthlyLabel", OBJPROP_CORNER, Corner);

   line++;

   ObjectCreate("TotalLabel", OBJ_LABEL, 0, 0.0, 0.0);
   ObjectSetText("TotalLabel", "Total   " + DoubleToStr(tp, 0) + " (" + DoubleToStr(tm, 2) + ")", FontSize, "Arial", TextColor);
   ObjectSet("TotalLabel", OBJPROP_XDISTANCE, PositionX);
   ObjectSet("TotalLabel", OBJPROP_YDISTANCE, PositionY + line * 1.5 * FontSize);
   ObjectSet("TotalLabel", OBJPROP_CORNER, Corner);
   
   return(0);
}

// ---------------------------------------------------------------------------------------------
// Function FindOrderTicket()
// ---------------------------------------------------------------------------------------------

int FindOrderTicket(int ticket, int start)
{
   for (int i = start; i < orderCount; i++)
   {
      if (orderTickets[i] == ticket)
      {
         return (i);
      }
   }
   return (-1);
}

// ---------------------------------------------------------------------------------------------
// Function AddOrderPoints()
// ---------------------------------------------------------------------------------------------

void AddOrderPoints(double pips, double profit)
{
   int dow = TimeDayOfWeek(TimeCurrent());
   int doy = TimeDayOfYear(TimeCurrent());
   int d = TimeDay(TimeCurrent());
   int m = TimeMonth(TimeCurrent());
   int y = TimeYear(TimeCurrent());
   
   // If the order close time is 0 the order is still open
   if (OrderCloseTime() == 0)
   {
      if (Symbol() == OrderSymbol())
      {
         sp += pips;
         sm += profit;
      }
   
      dp += pips;
      dm += profit;
      
      wp += pips;
      wm += profit;
      
      mp += pips;
      mm += profit;
         
   }
   else
   {   
      int odoy = TimeDayOfYear(OrderCloseTime());
      int od = TimeDay(OrderCloseTime());
      int om = TimeMonth(OrderCloseTime());
      int oy = TimeYear(OrderCloseTime());
   
      if (oy == y && om == m && od == d)
      {
         dp += pips;
         dm += profit;
      }
      
      if (oy == y && om == m && doy - odoy <= dow)
      {
         wp += pips;
         wm += profit;
      }
      
      if (oy == y && om == m)
      {
         mp += pips;
         mm += profit;
      }
   }
      
   tp += pips;
   tm += profit;   
}

